@@ -13,7 +13,8 @@
 
 createStamp <- function(script,file,time,addto){
 
-    if(missing(time)||is.null(time)) time <- Sys.time()
+    if(missing(time)||is.null(time)||(is.logical(time)&&time)) time <- Sys.time()
+    if(is.logical(time)&&!time) time <- ""
     if(missing(addto)) addto <- NULL
     if(inherits(time,"POSIXt")){
         time <- format(time, "%d-%b-%Y %H:%M")
